﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using Dynamicweb.Extensibility;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Content;

namespace Dynamicweb.Ecommerce.LiveIntegration.NotificationSubscribers
{
    [Subscribe(Dynamicweb.Ecommerce.Notifications.Ecommerce.Cart.CheckoutDone)]
    public class CheckoutDone : NotificationSubscriber
    {
        public override void OnNotify(string notification, NotificationArgs args)
        {
            if (!Configuration.Global.IsIntegrationActive)
                return;

            if (args != null)
            {
                var myArgs = (Dynamicweb.Ecommerce.Notifications.Ecommerce.Cart.CheckoutDoneArgs)args;

                if (myArgs != null && UrlHandler.Instance.IsWebServiceUrlFound() && IsEcomShopAllowed(myArgs.Order))
                {
                    if(myArgs.Order != null && myArgs.Order.Complete && myArgs.Order.Currency != Ecommerce.Common.Context.Currency)
                    {
                        //this can happen when redirected back from some CheckOut handler
                        //Common.Context.Currency can be set to the default Application currency, 
                        //while the completed order can be in another currency
                        Ecommerce.Common.Context.Currency = myArgs.Order.Currency;
                    }
                    OrderHandler.UpdateOrder(myArgs.Order);
                }
            }
        }

        private bool IsEcomShopAllowed(Dynamicweb.Ecommerce.Orders.Order order)
        {
            bool ret = true;
            string shopID = Helpers.GetConfigValue(Constants.WebServiceConfiguration.Path, Constants.WebServiceConfiguration.Shop);
            if (!string.IsNullOrEmpty(shopID))
            {
                if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
                {
                    if ((string)Dynamicweb.Frontend.PageView.Current().Area.EcomShopId != shopID)
                    {
                        ret = false;
                    }
                }
                else
                {
                    IPageService pageService = ServiceLocator.Current.GetPageService();
                    if (pageService != null)
                    {
                        Page page = pageService.GetPage(order.CheckoutPageId);
                        if (page != null)
                        {
                            IAreaService areaService = ServiceLocator.Current.GetAreaService();
                            if (areaService != null)
                            {
                                Area area = areaService.GetArea(page.AreaId);
                                if (area != null)
                                {
                                    if (area.EcomShopId != shopID)
                                    {
                                        ret = false;
                                    }
                                    else
                                    {
                                        // add shopid for UrlHandler to find the correct url
                                        System.Web.HttpContext.Current.Items.Add("shopid", area.EcomShopId);
                                    }
                                }
                            }
                        }
                    }
                }                
            }

            return ret;
        }        
    }
}
